@@ -44,6 +44,15 @@ extern "C" {
 #define DECOR_BLUR_ATOM_NAME                    "_COMPIZ_WM_WINDOW_BLUR_DECOR"
 #define DECOR_SWITCH_WINDOW_ATOM_NAME           "_COMPIZ_SWITCH_SELECT_WINDOW"
 #define DECOR_SWITCH_FOREGROUND_COLOR_ATOM_NAME "_COMPIZ_SWITCH_FOREGROUND_COLOR"
+#define DECOR_INPUT_FRAME_ATOM_NAME             "_COMPIZ_WINDOW_DECOR_INPUT_FRAME"
+#define DECOR_OUTPUT_FRAME_ATOM_NAME            "_COMPIZ_WINDOW_DECOR_OUTPUT_FRAME"
+
+#define DECOR_TYPE_ATOM_NAME                    "_COMPIZ_WINDOW_DECOR_TYPE"
+#define DECOR_TYPE_PIXMAP_ATOM_NAME             "_COMPIZ_WINDOW_DECOR_TYPE_PIXMAP"
+#define DECOR_TYPE_WINDOW_ATOM_NAME             "_COMPIZ_WINDOW_DECOR_TYPE_WINDOW"
+
+#define WINDOW_DECORATION_TYPE_PIXMAP (1 << 0)
+#define WINDOW_DECORATION_TYPE_WINDOW (1 << 1)
 
 #define GRAVITY_WEST  (1 << 0)
 #define GRAVITY_EAST  (1 << 1)
@@ -71,6 +80,11 @@ extern "C" {
 #define PAD_LEFT   (1 << 2)
 #define PAD_RIGHT  (1 << 3)
 
+#define BORDER_TOP    0
+#define BORDER_BOTTOM 1
+#define BORDER_LEFT   2
+#define BORDER_RIGHT  3
+
 typedef struct _decor_point {
     int x;
     int y;
@@ -160,7 +174,8 @@ typedef void (*decor_draw_func_t) (Display	   *xdisplay,
 				   decor_context_t *context,
 				   void		   *closure);
 
-#define BASE_PROP_SIZE 12
+#define WINDOW_PROP_SIZE 12
+#define BASE_PROP_SIZE 13
 #define QUAD_PROP_SIZE 9
 #define N_QUADS_MAX    24
 
@@ -177,18 +192,36 @@ decor_quads_to_property (long		 *data,
 			 decor_quad_t    *quad,
 			 int		 nQuad);
 
+void
+decor_gen_window_property (long		   *data,
+			   decor_extents_t *input,
+			   decor_extents_t *max_input,
+			   int		   min_width,
+			   int		   min_height);
+
 int
 decor_property_get_version (long *data);
 
 int
-decor_property_to_quads (long		 *data,
-			 int		 size,
-			 Pixmap		 *pixmap,
-			 decor_extents_t *input,
-			 decor_extents_t *max_input,
-			 int		 *min_width,
-			 int		 *min_height,
-			 decor_quad_t    *quad);
+decor_property_get_type (long *data);
+
+int
+decor_pixmap_property_to_quads (long		 *data,
+				int		 size,
+				Pixmap		 *pixmap,
+				decor_extents_t *input,
+				decor_extents_t *max_input,
+				int		 *min_width,
+				int		 *min_height,
+				decor_quad_t    *quad);
+
+int
+decor_window_property (long	       *data,
+		       int	       size,
+		       decor_extents_t *input,
+		       decor_extents_t *max_input,
+		       int	       *min_width,
+		       int	       *min_height);
 
 void
 decor_region_to_blur_property (long   *data,
@@ -346,52 +379,18 @@ decor_blend_transform_picture (Display	       *xdisplay,
 			       int	       shade_alpha);
 
 void
-decor_blend_top_border_picture (Display	        *xdisplay,
-				decor_context_t *context,
-				Picture	        src,
-				int	        xSrc,
-				int	        ySrc,
-				Picture	        dst,
-				decor_layout_t  *layout,
-				Region	        region,
-				unsigned short  alpha,
-				int	        shade_alpha);
-
-void
-decor_blend_bottom_border_picture (Display	   *xdisplay,
-				   decor_context_t *context,
-				   Picture	   src,
-				   int	           xSrc,
-				   int	           ySrc,
-				   Picture	   dst,
-				   decor_layout_t  *layout,
-				   Region	   region,
-				   unsigned short  alpha,
-				   int	           shade_alpha);
-
-void
-decor_blend_left_border_picture (Display	 *xdisplay,
-				 decor_context_t *context,
-				 Picture	 src,
-				 int	         xSrc,
-				 int	         ySrc,
-				 Picture	 dst,
-				 decor_layout_t  *layout,
-				 Region		 region,
-				 unsigned short  alpha,
-				 int	         shade_alpha);
-
-void
-decor_blend_right_border_picture (Display	  *xdisplay,
-				  decor_context_t *context,
-				  Picture	  src,
-				  int	          xSrc,
-				  int	          ySrc,
-				  Picture	  dst,
-				  decor_layout_t  *layout,
-				  Region	  region,
-				  unsigned short  alpha,
-				  int	          shade_alpha);
+decor_blend_border_picture (Display	    *xdisplay,
+			    decor_context_t *context,
+			    Picture	    src,
+			    int	            xSrc,
+			    int	            ySrc,
+			    Picture	    dst,
+			    decor_layout_t  *layout,
+			    unsigned int    border,
+			    Region	    region,
+			    unsigned short  alpha,
+			    int	            shade_alpha,
+			    int             ignore_src_alpha);
 
 #define DECOR_ACQUIRE_STATUS_SUCCESS	      0
 #define DECOR_ACQUIRE_STATUS_FAILED	      1
@@ -406,7 +405,8 @@ decor_acquire_dm_session (Display    *xdisplay,
 
 void
 decor_set_dm_check_hint (Display *xdisplay,
-			 int	 screen);
+			 int	 screen,
+			 int     supports);
 
 #define DECOR_SELECTION_KEEP    0
 #define DECOR_SELECTION_GIVE_UP 1
