@@ -30,7 +30,7 @@
 
 #include <fixx11h.h>
 
-#include <KDE/Plasma/PanelSvg>
+#include <KDE/Plasma/FrameSvg>
 #include <KDE/Plasma/Theme>
 
 #include <kwindowsystem.h>
@@ -38,23 +38,27 @@
 #include <QString>
 #include <QPainter>
 
-KWD::Switcher::Switcher (WId parentId, WId id):
-mId (id)
+
+KWD::Switcher::Switcher (WId, WId id):
+    mId (id),
+    mX11Pixmap (0),
+    mX11BackgroundPixmap (0)
 {
     QPalette palette;
     long     prop[4];
     QColor   color;
     color = Plasma::Theme::defaultTheme ()->color (Plasma::Theme::TextColor);
 
-    mBackground = new Plasma::PanelSvg();
+    mBackground = new Plasma::FrameSvg ();
     mBackground->setImagePath ("dialogs/background");
-    mBackground->setEnabledBorders(Plasma::PanelSvg::AllBorders);
+    mBackground->setEnabledBorders (Plasma::FrameSvg::AllBorders);
 
-    mBorder.left   = mBackground->marginSize(Plasma::LeftMargin);
-    mBorder.right  = mBackground->marginSize(Plasma::RightMargin);
-    mBorder.top    = mBackground->marginSize(Plasma::TopMargin);
-    mBorder.bottom = mBackground->marginSize(Plasma::BottomMargin) +
-		     Plasma::Theme::defaultTheme ()->fontMetrics ().height () + 10;
+    mBorder.left   = mBackground->marginSize (Plasma::LeftMargin);
+    mBorder.right  = mBackground->marginSize (Plasma::RightMargin);
+    mBorder.top    = mBackground->marginSize (Plasma::TopMargin);
+    mBorder.bottom = mBackground->marginSize (Plasma::BottomMargin) +
+		     Plasma::Theme::defaultTheme ()->fontMetrics ().height () +
+		     10;
 
     mContext.extents.left   = mBorder.left;
     mContext.extents.right  = mBorder.right;
@@ -86,6 +90,10 @@ mId (id)
 
 KWD::Switcher::~Switcher ()
 {
+    if (mX11Pixmap)
+	XFreePixmap (QX11Info::display (), mX11Pixmap);
+    if (mX11BackgroundPixmap)
+	XFreePixmap (QX11Info::display (), mX11BackgroundPixmap);
     delete mBackground;
 }
 
@@ -104,8 +112,29 @@ KWD::Switcher::updateGeometry ()
     KWD::readWindowProperty (mId, Atoms::switchSelectWindow,
 			     (long *)&mSelected);
 
+    if (mX11Pixmap)
+	XFreePixmap (QX11Info::display (), mX11Pixmap);
+    if (mX11BackgroundPixmap)
+	XFreePixmap (QX11Info::display (), mX11BackgroundPixmap);
+
+#ifdef QT_45
+    mX11Pixmap = XCreatePixmap (QX11Info::display (),
+				QX11Info::appRootWindow (),
+				width + mBorder.left + mBorder.right,
+				height + mBorder.top + mBorder.bottom, 32);
+
+    mX11BackgroundPixmap = XCreatePixmap (QX11Info::display (),
+					  QX11Info::appRootWindow (),
+					  width, height, 32);
+
+    mPixmap = QPixmap::fromX11Pixmap (mX11Pixmap, QPixmap::ExplicitlyShared);
+    mBackgroundPixmap = QPixmap::fromX11Pixmap (mX11BackgroundPixmap,
+						QPixmap::ExplicitlyShared);
+#else
     mPixmap = QPixmap (width + mBorder.left + mBorder.right,
 		       height + mBorder.top + mBorder.bottom);
+    mBackgroundPixmap = QPixmap (width, height);
+#endif
 
     redrawPixmap ();
     update ();
@@ -122,6 +151,7 @@ void
 KWD::Switcher::redrawPixmap ()
 {
     QPainter p (&mPixmap);
+    QPainter bp (&mBackgroundPixmap);
 
     const int contentWidth  = mPixmap.width ();
     const int contentHeight = mPixmap.height ();
@@ -131,16 +161,19 @@ KWD::Switcher::redrawPixmap ()
     p.setCompositionMode (QPainter::CompositionMode_Source);
     p.setRenderHint (QPainter::SmoothPixmapTransform);
 
-    mBackground->resizePanel (QSizeF (contentWidth, contentHeight));
-    mBackground->paintPanel (&p, QRect (0, 0, contentWidth,
-			contentHeight));
+    mBackground->resizeFrame (QSizeF (contentWidth, contentHeight));
+    mBackground->paintFrame (&p, QRect (0, 0, contentWidth, contentHeight));
 
-    mBackgroundPixmap = mPixmap.copy (mBorder.left, mBorder.top,
-				      mGeometry.width (),
-				      mGeometry.height ());
+    bp.drawPixmap (0, 0, mPixmap, mBorder.left, mBorder.top,
+		   mGeometry.width (), mGeometry.height ());
 
     XSetWindowBackgroundPixmap (QX11Info::display (), mId,
-				mBackgroundPixmap.handle ());
+#ifdef QT_45
+				mX11BackgroundPixmap);
+#else
+                                mBackgroundPixmap.handle ());
+#endif
+
     XClearWindow (QX11Info::display (), mId);
 }
 
@@ -168,12 +201,12 @@ KWD::Switcher::update ()
     p.setCompositionMode (QPainter::CompositionMode_Source);
 
 
-    mBackground->paintPanel (&p, QRect (mBorder.left, mBorder.top +
-			mGeometry.height () + 5, mGeometry.width (),
-			fm.height ()));
+    mBackground->paintFrame (&p, QRect (mBorder.left, mBorder.top +
+			     mGeometry.height () + 5, mGeometry.width (),
+			     fm.height ()));
 
     p.setFont (font);
-    p.setPen (Plasma::Theme::defaultTheme ()->color(Plasma::Theme::TextColor));
+    p.setPen (Plasma::Theme::defaultTheme ()->color (Plasma::Theme::TextColor));
 
     p.drawText ((mPixmap.width () - fm.width (name)) / 2,
                 mBorder.top + mGeometry.height () + 5 + fm.ascent (), name);
@@ -196,14 +229,19 @@ KWD::Switcher::updateWindowProperties ()
 
     nQuad = decor_set_lXrXtXbX_window_quads (quads, &mContext, &mDecorLayout,
 					     lh / 2, rh / 2, w, w / 2);
-
+#ifdef QT_45
+    decor_quads_to_property (data, mX11Pixmap,
+			     &mBorder, &mBorder,
+			     0, 0,
+			     quads, nQuad);
+#else
     decor_quads_to_property (data, mPixmap.handle (),
 			     &mBorder, &mBorder,
 			     0, 0,
 			     quads, nQuad);
-
+#endif
     KWD::trapXError ();
-    XChangeProperty (QX11Info::display(), mId, Atoms::netWindowDecor,
+    XChangeProperty (QX11Info::display (), mId, Atoms::netWindowDecor,
 		     XA_INTEGER, 32, PropModeReplace, (unsigned char *) data,
 		     BASE_PROP_SIZE + QUAD_PROP_SIZE * nQuad);
     KWD::popXError ();
@@ -232,7 +270,7 @@ KWD::Switcher::updateBlurProperty (int topOffset,
     if (blurType != BLUR_TYPE_NONE)
     {
 	QRegion r;
-	
+
 	topQRegion    = QRegion (-mContext.extents.left, -mContext.extents.top,
 				 w, mContext.extents.top);
 	topRegion     = topQRegion.handle ();
