@@ -44,20 +44,41 @@ decor_version (void)
 
   data[0] = version
 
-  data[1] = pixmap
+  data[1] = decoration type
 
-  data[2] = input left
-  data[3] = input right
-  data[4] = input top
-  data[5] = input bottom
+  WINDOW_DECORATION_TYPE_WINDOW property
+  --------------------------------------
+  data[3] = input left
+  data[4] = input right
+  data[5] = input top
+  data[6] = input bottom
 
-  data[6] = input left when maximized
-  data[7] = input right when maximized
-  data[8] = input top when maximized
-  data[9] = input bottom when maximized
+  data[7]  = input left when maximized
+  data[8]  = input right when maximized
+  data[9]  = input top when maximized
+  data[10] = input bottom when maximized
 
-  data[10] = min width
-  data[11] = min height
+  data[11] = min width
+  data[12] = min height
+
+
+  WINDOW_DECORATION_TYPE_PIXMAP property
+  --------------------------------------
+
+  data[2] = pixmap
+
+  data[3] = input left
+  data[4] = input right
+  data[5] = input top
+  data[6] = input bottom
+
+  data[7]  = input left when maximized
+  data[8]  = input right when maximized
+  data[9]  = input top when maximized
+  data[10] = input bottom when maximized
+
+  data[11] = min width
+  data[12] = min height
 
   flags
 
@@ -65,15 +86,15 @@ decor_version (void)
   9rd and 10th bit alignment, 11rd and 12th bit clamp,
   13th bit XX, 14th bit XY, 15th bit YX, 16th bit YY.
 
-  data[11 + n * 9 + 1] = flags
-  data[11 + n * 9 + 2] = p1 x
-  data[11 + n * 9 + 3] = p1 y
-  data[11 + n * 9 + 4] = p2 x
-  data[11 + n * 9 + 5] = p2 y
-  data[11 + n * 9 + 6] = widthMax
-  data[11 + n * 9 + 7] = heightMax
-  data[11 + n * 9 + 8] = x0
-  data[11 + n * 9 + 9] = y0
+  data[12 + n * 9 + 1] = flags
+  data[12 + n * 9 + 2] = p1 x
+  data[12 + n * 9 + 3] = p1 y
+  data[12 + n * 9 + 4] = p2 x
+  data[12 + n * 9 + 5] = p2 y
+  data[12 + n * 9 + 6] = widthMax
+  data[12 + n * 9 + 7] = heightMax
+  data[12 + n * 9 + 8] = x0
+  data[12 + n * 9 + 9] = y0
  */
 void
 decor_quads_to_property (long		 *data,
@@ -86,6 +107,7 @@ decor_quads_to_property (long		 *data,
 			 int		 nQuad)
 {
     *data++ = DECOR_INTERFACE_VERSION;
+    *data++ = WINDOW_DECORATION_TYPE_PIXMAP;
 
     memcpy (data++, &pixmap, sizeof (Pixmap));
 
@@ -128,6 +150,30 @@ decor_quads_to_property (long		 *data,
     }
 }
 
+void
+decor_gen_window_property (long		   *data,
+			   decor_extents_t *input,
+			   decor_extents_t *max_input,
+			   int		   min_width,
+			   int		   min_height)
+{
+    *data++ = DECOR_INTERFACE_VERSION;
+    *data++ = WINDOW_DECORATION_TYPE_WINDOW;
+
+    *data++ = input->left;
+    *data++ = input->right;
+    *data++ = input->top;
+    *data++ = input->bottom;
+
+    *data++ = max_input->left;
+    *data++ = max_input->right;
+    *data++ = max_input->top;
+    *data++ = max_input->bottom;
+
+    *data++ = min_width;
+    *data++ = min_height;
+}
+
 int
 decor_property_get_version (long *data)
 {
@@ -135,14 +181,20 @@ decor_property_get_version (long *data)
 }
 
 int
-decor_property_to_quads (long		 *data,
-			 int		 size,
-			 Pixmap		 *pixmap,
-			 decor_extents_t *input,
-			 decor_extents_t *max_input,
-			 int		 *min_width,
-			 int		 *min_height,
-			 decor_quad_t    *quad)
+decor_property_get_type (long *data)
+{
+    return (int) data[1];
+}
+
+int
+decor_pixmap_property_to_quads (long		 *data,
+				int		 size,
+				Pixmap		 *pixmap,
+				decor_extents_t *input,
+				decor_extents_t *max_input,
+				int		 *min_width,
+				int		 *min_height,
+				decor_quad_t    *quad)
 {
     int i, n, flags;
 
@@ -152,6 +204,10 @@ decor_property_to_quads (long		 *data,
     if (decor_property_get_version (data) != decor_version ())
 	return 0;
 
+    if (decor_property_get_type (data) != WINDOW_DECORATION_TYPE_PIXMAP)
+	return 0;
+
+    data++;
     data++;
 
     memcpy (pixmap, data++, sizeof (Pixmap));
@@ -204,6 +260,42 @@ decor_property_to_quads (long		 *data,
     return n;
 }
 
+int
+decor_window_property (long	       *data,
+		       int	       size,
+		       decor_extents_t *input,
+		       decor_extents_t *max_input,
+		       int	       *min_width,
+		       int	       *min_height)
+{
+    if (size < WINDOW_PROP_SIZE)
+	return 0;
+
+    if (decor_property_get_version (data) != decor_version ())
+	return 0;
+
+    if (decor_property_get_type (data) != WINDOW_DECORATION_TYPE_WINDOW)
+	return 0;
+
+    data++;
+    data++;
+
+    input->left   = *data++;
+    input->right  = *data++;
+    input->top    = *data++;
+    input->bottom = *data++;
+
+    max_input->left   = *data++;
+    max_input->right  = *data++;
+    max_input->top    = *data++;
+    max_input->bottom = *data++;
+
+    *min_width  = *data++;
+    *min_height = *data++;
+
+    return 1;
+}
+
 static int
 add_blur_boxes (long   *data,
 		BoxPtr box,
@@ -1665,7 +1757,7 @@ decor_fill_picture_extents_with_shadow (Display	        *xdisplay,
 					Picture	        picture,
 					decor_layout_t  *layout)
 {
-    int w, h, x2, y2, left, right, top, bottom, width, height;
+    int w, h, left, right, top, bottom, width, height;
 
     if (!shadow->picture)
 	return;
@@ -1698,9 +1790,6 @@ decor_fill_picture_extents_with_shadow (Display	        *xdisplay,
     w = width - left - right;
     h = height - top - bottom;
 
-    x2 = width - right;
-    y2 = height - bottom;
-
     /* top left */
     XRenderComposite (xdisplay, PictOpSrc, shadow->picture, 0, picture,
 		      0, 0,
@@ -2064,13 +2153,15 @@ _decor_blend_horz_border_picture (Display	  *xdisplay,
 				  int		  x2,
 				  int		  y2,
 				  int		  dy,
-				  int		  direction)
+				  int		  direction,
+				  int             ignore_src_alpha)
 {
     XRenderColor color[3] = {
 	{ 0xffff, 0xffff, 0xffff, 0xffff },
-	{  alpha,  alpha,  alpha,  alpha }
+	{  alpha,  alpha,  alpha,  alpha },
+	{    0x0,    0x0,    0x0, 0xffff }
     };
-    int		 op = PictOpSrc;
+    int		 op = PictOpSrc, gop = PictOpSrc;
     int		 left, right;
 
     left   = context->extents.left;
@@ -2080,6 +2171,18 @@ _decor_blend_horz_border_picture (Display	  *xdisplay,
     XRenderSetPictureClipRegion (xdisplay, dst, region);
     XOffsetRegion (region, -x1, -y1);
 
+    if (ignore_src_alpha)
+    {
+	XRenderComposite (xdisplay, PictOpSrc, src, None, dst,
+			  xSrc, ySrc,
+			  0, 0,
+			  x1, y1,
+			  x2 - x1, y2 - y1);
+	XRenderFillRectangle (xdisplay, PictOpAdd, dst, &color[2], x1, y1,
+			      x2 - x1, y2 - y1);
+	gop = PictOpInReverse;
+    }
+
     if (alpha != 0xffff)
     {
 	op = PictOpIn;
@@ -2125,7 +2228,7 @@ _decor_blend_horz_border_picture (Display	  *xdisplay,
 	    XRenderSetPictureTransform (xdisplay, grad, &transform);
 	    XRenderChangePicture (xdisplay, grad, CPRepeat, &attrib);
 
-	    XRenderComposite (xdisplay, PictOpSrc, grad, None, dst,
+	    XRenderComposite (xdisplay, gop, grad, None, dst,
 			      0, 0,
 			      0, 0,
 			      x1, y1,
@@ -2156,7 +2259,7 @@ _decor_blend_horz_border_picture (Display	  *xdisplay,
 
 	    XRenderChangePicture (xdisplay, grad, CPRepeat, &attrib);
 
-	    XRenderComposite (xdisplay, PictOpSrc, grad, None, dst,
+	    XRenderComposite (xdisplay, gop, grad, None, dst,
 			      0, 0,
 			      0, 0,
 			      x1 + left, y1,
@@ -2182,7 +2285,7 @@ _decor_blend_horz_border_picture (Display	  *xdisplay,
 	    XRenderSetPictureTransform (xdisplay, grad, &transform);
 	    XRenderChangePicture (xdisplay, grad, CPRepeat, &attrib);
 
-	    XRenderComposite (xdisplay, PictOpSrc, grad, None, dst,
+	    XRenderComposite (xdisplay, gop, grad, None, dst,
 			      0, 0,
 			      0, 0,
 			      x2 - right, y1,
@@ -2192,108 +2295,21 @@ _decor_blend_horz_border_picture (Display	  *xdisplay,
 	}
 	else
 	{
-	    XRenderFillRectangle (xdisplay, PictOpSrc, dst, &color[1],
+	    XRenderFillRectangle (xdisplay, gop, dst, &color[1],
 				  x1, y1, x2 - x1, y2 - y1);
 	}
     }
 
-    XRenderComposite (xdisplay, op, src, None, dst,
-		      xSrc, ySrc,
-		      0, 0,
-		      x1, y1,
-		      x2 - x1, y2 - y1);
+    if (!ignore_src_alpha)
+	XRenderComposite (xdisplay, op, src, None, dst,
+			  xSrc, ySrc,
+			  0, 0,
+			  x1, y1,
+			  x2 - x1, y2 - y1);
 
     set_no_picture_clip (xdisplay, dst);
 }
 
-void
-decor_blend_top_border_picture (Display	        *xdisplay,
-				decor_context_t *context,
-				Picture	        src,
-				int	        xSrc,
-				int	        ySrc,
-				Picture	        dst,
-				decor_layout_t  *layout,
-				Region	        region,
-				unsigned short  alpha,
-				int	        shade_alpha)
-{
-    int left, right, top;
-    int x1, y1, x2, y2;
-
-    left  = context->extents.left;
-    right = context->extents.right;
-    top   = context->extents.top;
-
-    x1 = layout->top.x1 + context->left_space - left;
-    y1 = layout->top.y1 + context->top_space - top;
-    x2 = layout->top.x2 - context->right_space + right;
-    y2 = layout->top.y2;
-
-    _decor_blend_horz_border_picture (xdisplay,
-				      context,
-				      src,
-				      xSrc,
-				      ySrc,
-				      dst,
-				      layout,
-				      region,
-				      alpha,
-				      shade_alpha,
-				      x1,
-				      y1,
-				      x2,
-				      y2,
-				      top,
-				      -1);
-
-    _decor_pad_border_picture (xdisplay, dst, &layout->top);
-}
-
-void
-decor_blend_bottom_border_picture (Display	   *xdisplay,
-				   decor_context_t *context,
-				   Picture	   src,
-				   int	           xSrc,
-				   int	           ySrc,
-				   Picture	   dst,
-				   decor_layout_t  *layout,
-				   Region	   region,
-				   unsigned short  alpha,
-				   int	           shade_alpha)
-{
-    int left, right, bottom;
-    int x1, y1, x2, y2;
-
-    left   = context->extents.left;
-    right  = context->extents.right;
-    bottom = context->extents.bottom;
-
-    x1 = layout->bottom.x1 + context->left_space - left;
-    y1 = layout->bottom.y1;
-    x2 = layout->bottom.x2 - context->right_space + right;
-    y2 = layout->bottom.y1 + bottom;
-
-    _decor_blend_horz_border_picture (xdisplay,
-				      context,
-				      src,
-				      xSrc,
-				      ySrc,
-				      dst,
-				      layout,
-				      region,
-				      alpha,
-				      shade_alpha,
-				      x1,
-				      y1,
-				      x2,
-				      y2,
-				      bottom,
-				      1);
-
-    _decor_pad_border_picture (xdisplay, dst, &layout->bottom);
-}
-
 static void
 _decor_blend_vert_border_picture (Display	  *xdisplay,
 				  decor_context_t *context,
@@ -2309,13 +2325,15 @@ _decor_blend_vert_border_picture (Display	  *xdisplay,
 				  int		  y1,
 				  int		  x2,
 				  int		  y2,
-				  int		  direction)
+				  int		  direction,
+				  int             ignore_src_alpha)
 {
     XRenderColor color[3] = {
 	{ 0xffff, 0xffff, 0xffff, 0xffff },
-	{  alpha,  alpha,  alpha,  alpha }
+	{  alpha,  alpha,  alpha,  alpha },
+	{    0x0,    0x0,    0x0, 0xffff }
     };
-    int		 op = PictOpSrc;
+    int		 op = PictOpSrc, gop = PictOpSrc;
 
     if (layout->rotation)
     {
@@ -2348,6 +2366,44 @@ _decor_blend_vert_border_picture (Display	  *xdisplay,
 	XOffsetRegion (region, -x1, -y1);
     }
 
+    if (ignore_src_alpha)
+    {
+	if (layout->rotation)
+	{
+	    XTransform t = {
+		{
+		    {       0, 1 << 16,       0 },
+		    { 1 << 16,       0,       0 },
+		    {       0,       0, 1 << 16 }
+		}
+	    };
+
+	    t.matrix[0][2] = xSrc << 16;
+	    t.matrix[1][2] = ySrc << 16;
+
+	    XRenderSetPictureTransform (xdisplay, src, &t);
+
+	    XRenderComposite (xdisplay, PictOpSrc, src, None, dst,
+			      0, 0,
+			      0, 0,
+			      x1, y1, x2 - x1, y2 - y1);
+	    XRenderFillRectangle (xdisplay, PictOpAdd, dst, &color[2], x1, y1,
+			          x2 - x1, y2 - y1);
+
+	    XRenderSetPictureTransform (xdisplay, src, &xident);
+	}
+	else
+	{
+	    XRenderComposite (xdisplay, PictOpSrc, src, None, dst,
+			      xSrc, ySrc,
+			      0, 0,
+			      x1, y1, x2 - x1, y2 - y1);
+	    XRenderFillRectangle (xdisplay, PictOpAdd, dst, &color[2], x1, y1,
+			      x2 - x1, y2 - y1);
+	}
+	gop = PictOpInReverse;
+    }
+
     if (alpha != 0xffff)
     {
 	op = PictOpIn;
@@ -2402,7 +2458,7 @@ _decor_blend_vert_border_picture (Display	  *xdisplay,
 
 	    XRenderChangePicture (xdisplay, grad, CPRepeat, &attrib);
 
-	    XRenderComposite (xdisplay, PictOpSrc, grad, None, dst,
+	    XRenderComposite (xdisplay, gop, grad, None, dst,
 			      0, 0,
 			      0, 0,
 			      x1, y1,
@@ -2412,136 +2468,194 @@ _decor_blend_vert_border_picture (Display	  *xdisplay,
 	}
 	else
 	{
-	    XRenderFillRectangle (xdisplay, PictOpSrc, dst, &color[1],
+	    XRenderFillRectangle (xdisplay, gop, dst, &color[1],
 				  x1, y1, x2 - x1, y2 - y1);
 	}
     }
 
-    if (layout->rotation)
+    if (!ignore_src_alpha)
     {
-	XTransform t = {
-	    {
-		{       0, 1 << 16,       0 },
-		{ 1 << 16,       0,       0 },
-		{       0,       0, 1 << 16 }
-	    }
-	};
+	if (layout->rotation)
+	{
+	    XTransform t = {
+		{
+		    {       0, 1 << 16,       0 },
+		    { 1 << 16,       0,       0 },
+		    {       0,       0, 1 << 16 }
+		}
+	    };
 
-	t.matrix[0][2] = xSrc << 16;
-	t.matrix[1][2] = ySrc << 16;
+	    t.matrix[0][2] = xSrc << 16;
+	    t.matrix[1][2] = ySrc << 16;
 
-	XRenderSetPictureTransform (xdisplay, src, &t);
+	    XRenderSetPictureTransform (xdisplay, src, &t);
 
-	XRenderComposite (xdisplay, op, src, None, dst,
-			  0, 0,
-			  0, 0,
-			  x1, y1, x2 - x1, y2 - y1);
+	    XRenderComposite (xdisplay, op, src, None, dst,
+			    0, 0,
+			    0, 0,
+			    x1, y1, x2 - x1, y2 - y1);
 
-	XRenderSetPictureTransform (xdisplay, src, &xident);
-    }
-    else
-    {
-	XRenderComposite (xdisplay, op, src, None, dst,
-			  xSrc, ySrc,
-			  0, 0,
-			  x1, y1, x2 - x1, y2 - y1);
+	    XRenderSetPictureTransform (xdisplay, src, &xident);
+	}
+	else
+	{
+	    XRenderComposite (xdisplay, op, src, None, dst,
+			    xSrc, ySrc,
+			    0, 0,
+			    x1, y1, x2 - x1, y2 - y1);
+	}
     }
 
     set_no_picture_clip (xdisplay, dst);
 }
 
 void
-decor_blend_left_border_picture (Display	 *xdisplay,
-				 decor_context_t *context,
-				 Picture	 src,
-				 int	         xSrc,
-				 int	         ySrc,
-				 Picture	 dst,
-				 decor_layout_t  *layout,
-				 Region		 region,
-				 unsigned short  alpha,
-				 int	         shade_alpha)
+decor_blend_border_picture (Display	    *xdisplay,
+			    decor_context_t *context,
+			    Picture	    src,
+			    int	            xSrc,
+			    int	            ySrc,
+			    Picture	    dst,
+			    decor_layout_t  *layout,
+			    unsigned int    border,
+			    Region	    region,
+			    unsigned short  alpha,
+			    int	            shade_alpha,
+			    int             ignore_src_alpha)
 {
+    int left, right, bottom, top;
     int x1, y1, x2, y2;
 
-    x1 = layout->left.x1;
-    y1 = layout->left.y1;
-    x2 = layout->left.x2;
-    y2 = layout->left.y2;
+    left   = context->extents.left;
+    right  = context->extents.right;
+    top    = context->extents.top;
+    bottom = context->extents.bottom;
 
-    if (layout->rotation)
-	y1 += context->left_space - context->extents.left;
-    else
-	x1 += context->left_space - context->extents.left;
+    switch (border)
+    {
+    case BORDER_TOP:
+	x1 = layout->top.x1 + context->left_space - left;
+	y1 = layout->top.y1 + context->top_space - top;
+	x2 = layout->top.x2 - context->right_space + right;
+	y2 = layout->top.y2;
 
-    _decor_blend_vert_border_picture (xdisplay,
-				      context,
-				      src,
-				      xSrc,
-				      ySrc,
-				      dst,
-				      layout,
-				      region,
-				      alpha,
-				      shade_alpha,
-				      x1,
-				      y1,
-				      x2,
-				      y2,
-				      1);
+	_decor_blend_horz_border_picture (xdisplay,
+					context,
+					src,
+					xSrc,
+					ySrc,
+					dst,
+					layout,
+					region,
+					alpha,
+					shade_alpha,
+					x1,
+					y1,
+					x2,
+					y2,
+					top,
+					-1,
+					ignore_src_alpha);
 
-    _decor_pad_border_picture (xdisplay, dst, &layout->left);
-}
+	_decor_pad_border_picture (xdisplay, dst, &layout->top);
+	break;
+    case BORDER_BOTTOM:
+	x1 = layout->bottom.x1 + context->left_space - left;
+	y1 = layout->bottom.y1;
+	x2 = layout->bottom.x2 - context->right_space + right;
+	y2 = layout->bottom.y1 + bottom;
 
-void
-decor_blend_right_border_picture (Display	  *xdisplay,
-				  decor_context_t *context,
-				  Picture	  src,
-				  int	          xSrc,
-				  int	          ySrc,
-				  Picture	  dst,
-				  decor_layout_t  *layout,
-				  Region	  region,
-				  unsigned short  alpha,
-				  int	          shade_alpha)
-{
-    int x1, y1, x2, y2;
+	_decor_blend_horz_border_picture (xdisplay,
+					context,
+					src,
+					xSrc,
+					ySrc,
+					dst,
+					layout,
+					region,
+					alpha,
+					shade_alpha,
+					x1,
+					y1,
+					x2,
+					y2,
+					bottom,
+					1,
+					ignore_src_alpha);
 
-    x1 = layout->right.x1;
-    y1 = layout->right.y1;
-    x2 = layout->right.x2;
-    y2 = layout->right.y2;
+	_decor_pad_border_picture (xdisplay, dst, &layout->bottom);
+	break;
+    case BORDER_LEFT:
+	x1 = layout->left.x1;
+	y1 = layout->left.y1;
+	x2 = layout->left.x2;
+	y2 = layout->left.y2;
 
-    if (layout->rotation)
-	y2 -= context->right_space - context->extents.right;
-    else
-	x2 -= context->right_space - context->extents.right;
+	if (layout->rotation)
+	    y1 += context->left_space - context->extents.left;
+	else
+	    x1 += context->left_space - context->extents.left;
 
-    _decor_blend_vert_border_picture (xdisplay,
-				      context,
-				      src,
-				      xSrc,
-				      ySrc,
-				      dst,
-				      layout,
-				      region,
-				      alpha,
-				      shade_alpha,
-				      x1,
-				      y1,
-				      x2,
-				      y2,
-				      -1);
+	_decor_blend_vert_border_picture (xdisplay,
+					context,
+					src,
+					xSrc,
+					ySrc,
+					dst,
+					layout,
+					region,
+					alpha,
+					shade_alpha,
+					x1,
+					y1,
+					x2,
+					y2,
+					1,
+					ignore_src_alpha);
 
-    _decor_pad_border_picture (xdisplay, dst, &layout->right);
+	_decor_pad_border_picture (xdisplay, dst, &layout->left);
+	break;
+    case BORDER_RIGHT:
+	x1 = layout->right.x1;
+	y1 = layout->right.y1;
+	x2 = layout->right.x2;
+	y2 = layout->right.y2;
+
+	if (layout->rotation)
+	    y2 -= context->right_space - context->extents.right;
+	else
+	    x2 -= context->right_space - context->extents.right;
+
+	_decor_blend_vert_border_picture (xdisplay,
+					context,
+					src,
+					xSrc,
+					ySrc,
+					dst,
+					layout,
+					region,
+					alpha,
+					shade_alpha,
+					x1,
+					y1,
+					x2,
+					y2,
+					-1,
+					ignore_src_alpha);
+
+	_decor_pad_border_picture (xdisplay, dst, &layout->right);
+	break;
+    default:
+	break;
+    }
 }
 
 int
-decor_acquire_dm_session (Display *xdisplay,
-			  int	   screen,
-			  char     *name,
-			  int	   replace_current_dm,
-			  Time	   *timestamp)
+decor_acquire_dm_session (Display    *xdisplay,
+			  int	     screen,
+			  const char *name,
+			  int	     replace_current_dm,
+			  Time	     *timestamp)
 {
     XEvent		 event;
     XSetWindowAttributes attr;
@@ -2554,11 +2668,11 @@ decor_acquire_dm_session (Display *xdisplay,
     char		 buf[128];
 
     manager_atom = XInternAtom (xdisplay, "MANAGER", FALSE);
-    dm_name_atom = XInternAtom (xdisplay, "_NET_DM_NAME", 0);
+    dm_name_atom = XInternAtom (xdisplay, "_COMPIZ_DM_NAME", 0);
 
     utf8_string_atom = XInternAtom (xdisplay, "UTF8_STRING", 0);
 
-    sprintf (buf, "DM_S%d", screen);
+    sprintf (buf, "_COMPIZ_DM_S%d", screen);
     dm_sn_atom = XInternAtom (xdisplay, buf, 0);
 
     current_dm_sn_owner = XGetSelectionOwner (xdisplay, dm_sn_atom);
@@ -2637,12 +2751,25 @@ decor_acquire_dm_session (Display *xdisplay,
 
 void
 decor_set_dm_check_hint (Display *xdisplay,
-			 int	 screen)
+			 int	 screen,
+			 int     supports)
 {
     XSetWindowAttributes attrs;
     unsigned long	 data;
     Window		 xroot;
     Atom		 atom;
+    Atom		 type_pixmap_atom;
+    Atom		 type_window_atom;
+    Atom		 type_supported_atom;
+    Atom		 supported_deco_atoms[2];
+    int                  i;
+
+    type_supported_atom = XInternAtom (xdisplay, DECOR_TYPE_ATOM_NAME, 0);
+    type_pixmap_atom = XInternAtom (xdisplay, DECOR_TYPE_PIXMAP_ATOM_NAME, 0);
+    type_window_atom = XInternAtom (xdisplay, DECOR_TYPE_WINDOW_ATOM_NAME, 0);
+
+    if (!supports)
+	return;
 
     attrs.override_redirect = 1;
     attrs.event_mask	    = PropertyChangeMask;
@@ -2659,7 +2786,26 @@ decor_set_dm_check_hint (Display *xdisplay,
 			  CWOverrideRedirect | CWEventMask,
 			  &attrs);
 
-    atom = XInternAtom (xdisplay, "_NET_SUPPORTING_DM_CHECK", 0);
+    i = 0;
+    if (supports & WINDOW_DECORATION_TYPE_PIXMAP)
+    {
+	supported_deco_atoms[i] = type_pixmap_atom;
+	i++;
+    }
+    if (supports & WINDOW_DECORATION_TYPE_WINDOW)
+    {
+	supported_deco_atoms[i] = type_window_atom;
+	i++;
+    }
+    XChangeProperty (xdisplay,
+		     data,
+		     type_supported_atom,
+		     XA_ATOM, 32,
+		     PropModeReplace,
+		     (unsigned char *) supported_deco_atoms,
+		     i);
+
+    atom = XInternAtom (xdisplay, DECOR_SUPPORTING_DM_CHECK_ATOM_NAME, 0);
 
     XChangeProperty (xdisplay, xroot,
 		     atom,
@@ -2797,7 +2943,7 @@ decor_handle_selection_clear (Display *xdisplay,
     Atom dm_sn_atom;
     char buf[128];
 
-    sprintf (buf, "DM_S%d", screen);
+    sprintf (buf, "_COMPIZ_DM_S%d", screen);
     dm_sn_atom = XInternAtom (xdisplay, buf, 0);
 
     if (xevent->xselectionclear.selection == dm_sn_atom)
